define([
	"aps/i18n",
	"aps/Message",
	"aps/Output",
	"aps/ResourceStore",
	"dojo/Deferred",
	"dojo/_base/array",
	"aps/Tooltip",
	"dojo/date",
	"dojo/date/locale",
	"dijit/registry",
	"./displayError.js"
], function(
	_,
	Message,
	Output,
	Store,
	Deferred,
	array,
	Tooltip,
	dojo_date,
	locale,
	registry,
	displayError
) {
	"use strict";

	var utils = {
		showLoading: function() {
			if (aps.apsc.showLoading) {
				aps.apsc.showLoading();
			}
		},
		hideLoading: function() {
			if (aps.apsc.hideLoading) {
				aps.apsc.hideLoading();
			}
		},

		escapeHTML: function(text) {
			var output = new Output({ value: text });
			return output.value;
		},
		
		apsStatus: {
			ready: "aps:ready"
		},

		formatDate: function(date, pattern) {
			if (!date) return "";
			if (!pattern) pattern = "MMM-dd-yyyy";
			return locale.format(date, {datePattern: pattern, selector: "date"});
		},

		formatTime: function(date) {
			if (!date) return "";
			return locale.format(date, {datePattern: "dd MMM yyyy HH:mm", selector: "date"});
		},

		daysBetween: function(date1, date2) {
			var timeDiff = date2.getTime() - date1.getTime();
			var diffDays = Math.ceil(timeDiff / (24 * 3600 * 1000));
			return diffDays;
		},

		weekAgo: function(date){
			return dojo_date.add(date, "day", -6);
		},

		monthAgo: function(date){
			return dojo_date.add(date, "month", -1);
		},

		threeMonthsAgo: function(date){
			return dojo_date.add(date, "month", -3);
		},

		compareDates: function(date1, date2, portion){
			return dojo_date.compare(date1, date2, portion);
		},

		isEqualDates: function(date1, date2){
			var dateParams = function(date){
				return [date.getDate(), date.getMonth(), date.getFullYear()];
			};
			return dateParams(date1).toString() == dateParams(date2).toString();
		},

		formatUnit: function(unit) {
			if (unit.match(/[\d]/)){ // Unit of measure
				return "(" + unit + ")";
			}
			return unit;
		},

		getFullResource: function (res, property) {
			if (!!property && property in res) {
				var result = new Deferred();
				result.resolve(res);
				return result;
			} else {
				return new Store({
					target: "/aps/2/resources/" + res.aps.id + "/full"
				}).query();
			}
		},

		showTooltip: function(innerHTML, ancorControl, position){
			Tooltip.show(innerHTML, ancorControl, position, "", "", null, null);
		},

		hideTooltip: function(ancorControl){
			Tooltip.hide(ancorControl);
		},

		replaceOptionsInSelect: function(select, start, options){
			// TODO w/a for query and domConstruct
			while (select.length > start){
				select.remove(start);
			}
			array.forEach(options, function(option){
				var newOption = document.createElement("option");
				newOption.innerHTML = option.label;
				newOption.value = option.value;

				if (option.selected){
					newOption.selected = "selected";
				}
				select.add(newOption);
			});
		},

		arrayToString: function(arr){
			if (arr.length > 1){
				var last = arr.pop();
				return arr.join(", ") + _(" and ") + last;
			} else {
				return arr[0];
			}
		},

		getSearchRegexByValue: function(value){
			var regex = /[^*]+/g;
			var matches = [];
			var match;
			do {
				match = regex.exec(value);
				if (match) {
					matches.push("(" + match + ")");
				}
			} while (match);
			if (matches.length === 0){
				return new RegExp(value, "gi");
			}

			var starSubstitute = "[\\w\\W]*";
			var pattern = "^";
			if (value.charAt(0) === "*"){
				pattern += starSubstitute;
			}
			pattern += matches.join(starSubstitute);
			if (value.slice(-1) === "*"){
				pattern += starSubstitute;
			}
			pattern += "$";
			return new RegExp(pattern, "gi");
		},

		getSortByColumn: function(prop, descending){
			switch (prop) {
				case "name":
				case "region":
				case "category":
				case "subcategory":
				default:
					return function(a,b){
						return a[prop].localeCompare(b[prop]) * (descending ? -1 : 1);
					};
				case "rate":
				case "amount":
				case "billable":
				case "included":
				case "consumed":
					return function(a,b){
						return (a[prop] - b[prop]) * (descending ? -1 : 1);
					};
				case "reportedTime":
					return function(a,b){
						return utils.compareDates(a[prop], b[prop]) * (descending ? -1 : 1);
					};
			}
		},

		getUnspecified: function(){
			return {
				value: "UNSPECIFIED",
				label: _("Unspecified")
			};
		},

		isUnspecified: function(key){
			return !key || key === utils.getUnspecified().value;
		},

		getUnspecifiedPosibile: function(val){
			return utils.isUnspecified(val) ? utils.getUnspecified().label : val;
		},

		onErrorCallback: function(error){
			displayError(error);
		},

		getHumanReadableLastErrorRequest: function(lastError, codeMapper){
			var result = new Deferred();

			if (!lastError || !lastError.code) {
				result.resolve("");
				return result;
			}

			var isEmptyPoperties = (!lastError.properties || lastError.properties.length === 0);

			var getErrorMessage = function(error, def){
				var errMsg = error.getErrorMessage(lastError.properties);
				if (errMsg.then){
					return errMsg;
				} else {
					def.resolve({
						text: errMsg,
						type: error.type
					});
					return def;
				}
			};

			if (codeMapper.hasOwnProperty(lastError.code)){
				var error = codeMapper[lastError.code];
				if (error.needProperties){
					if (!isEmptyPoperties){
						return getErrorMessage(error, result);
					}
				} else {
					return getErrorMessage(error, result);
				}
			}

			var propertyNamesToString = function(propertyNames){
				return utils.arrayToString(array.map(propertyNames, function(property){
					return property.name + (property.value ? " \"" + property.value + "\"" : "");
				}));
			};

			var resultError = lastError.code;
			if (!isEmptyPoperties) {
				resultError += ": " + propertyNamesToString(lastError.properties);
			}
			result.resolve({
				text: resultError,
				type: "error"
			});

			return result;
		},

		updateErrorMessage: function(lastError, codeMapper, pageId){
			var page = registry.byId(pageId ? pageId : "pageContainer")
				, messages = page.get("messageList");

			var lastErrorMsg = registry.byId("lastError");
			if (lastErrorMsg) {
				messages.removeChild(lastErrorMsg);
			}

			if (lastError) {
				var lastErrorRequest = utils.getHumanReadableLastErrorRequest(lastError, codeMapper);
				lastErrorRequest.then(function(error){
					if (error && error.text) {
						messages.addChild(new Message({
							id: "lastError",
							description: error.text,
							type: error.type ? error.type : "error",
							closeable: false
						}));
					}
				}, utils.onErrorCallback);
			}
		},

		isNumeric: function(val){
			return !isNaN(parseFloat(val)) && isFinite(val);
		},

		getNoPriceMessage: function(){
			return _("Price is undefined for this resource. Please contact provider for details.");
		}
	};
	return utils;
});